@@ -81,8 +81,8 @@ struct	firewire_header {
 
 #define	FIREWIREMTU	(FIREWIRE_MAX_LEN-FIREWIRE_HDR_LEN)
 
-int firewire_attach_inet(ifnet_t ifp, u_long protocol_family);
-int firewire_attach_inet6(ifnet_t ifp, __unused u_long protocol_family);
+int firewire_attach_inet(ifnet_t ifp, protocol_family_t protocol_family);
+int firewire_attach_inet6(ifnet_t ifp, __unused protocol_family_t protocol_family);
 
 int firewire_ifattach(register ifnet_t ifp);
 void firewire_ifdetach(register ifnet_t ifp);
