@@ -47,7 +47,8 @@ bool IOFireWireIP::start(IOService *provider)
 	fMaxPktSize = 0;
 	netifEnabled = false;
 	busifEnabled = false;
-	
+	fDoFastRetry = false;
+
     fDevice = OSDynamicCast(IOFireWireNub, provider);
 
     if(!fDevice)
@@ -160,8 +161,8 @@ bool IOFireWireIP::start(IOService *provider)
 		CSRNodeUniqueID	fwuid = fDevice->getUniqueID();
 
 		memset(&hwAddr, 0, sizeof(IP1394_HDW_ADDR));
-		hwAddr.eui64.hi = (UInt32)(fwuid >> 32);
-		hwAddr.eui64.lo = (UInt32)(fwuid & 0xffffffff);
+		hwAddr.eui64.hi = OSSwapHostToBigInt32((UInt32)(fwuid >> 32));
+		hwAddr.eui64.lo = OSSwapHostToBigInt32((UInt32)(fwuid & 0xffffffff));
 		
 		hwAddr.maxRec	= fControl->getMaxRec();              
 		hwAddr.spd		= fDevice->FWSpeed();
@@ -171,6 +172,11 @@ bool IOFireWireIP::start(IOService *provider)
 		
 		memcpy(&fLcb->ownHardwareAddress, &hwAddr, sizeof(IP1394_HDW_ADDR)); 
 		
+		UInt32 size = getMaxARDMAPacketSize();
+		
+		if(size > 0)
+			fLcb->ownHardwareAddress.maxRec = getMaxARDMARec(size);			
+		
 		setProperty(kIOFWHWAddr,  (void *)&fLcb->ownHardwareAddress, sizeof(IP1394_HDW_ADDR));
 	}
 	
@@ -188,66 +194,66 @@ bool IOFireWireIP::start(IOService *provider)
 } // end start
 
 bool IOFireWireIP::finalize(IOOptionBits options)
-{
-	
+{	
 	return super::finalize(options);
 }
 
 void IOFireWireIP::stop(IOService *provider)
 {
 	if(fDiagnostics_Symbol != NULL)
-	{
 		fDiagnostics_Symbol->release();		
-		fDiagnostics_Symbol = 0;
-	}
-
-	if(fPolicyMaker != NULL)
-		fPolicyMaker->deRegisterInterestedDriver(this);
-
-    if (ipLock != NULL) 
-	{
-        IORecursiveLockFree(ipLock);
-        ipLock = NULL;
-    }
+	
+	fDiagnostics_Symbol = 0;
 
     // Free the firewire stuff
-    if (fLocalIP1394v6ConfigDirectory != NULL){
+    if (fLocalIP1394v6ConfigDirectory != NULL)
+	{
         // clear the unit directory in config rom
         fDevice->getBus()->RemoveUnitDirectory(fLocalIP1394v6ConfigDirectory) ;
 		fLocalIP1394v6ConfigDirectory->release();
 	} 
+	fLocalIP1394v6ConfigDirectory = NULL;
 
     // Free the firewire stuff
-    if (fLocalIP1394ConfigDirectory != NULL){
+    if (fLocalIP1394ConfigDirectory != NULL)
+	{
         // clear the unit directory in config rom
         fDevice->getBus()->RemoveUnitDirectory(fLocalIP1394ConfigDirectory) ;
 		fLocalIP1394ConfigDirectory->release();
 	} 
+	
+	fLocalIP1394ConfigDirectory = NULL;
 
 	if(fwOwnAddr != NULL)
 		fwOwnAddr->release();
 	
+	fwOwnAddr = NULL;
+	
 	if (transmitQueue != NULL)
 		transmitQueue->release();
 	
+	transmitQueue = NULL;
+	
 	if(fIPv6UnitNotifier != NULL)
-	{
 		fIPv6UnitNotifier->remove();
-		fIPv6UnitNotifier = NULL;
-	}
+	
+	fIPv6UnitNotifier = NULL;
 	
 	// Remove IOFireWireIPUnit notification
 	if(fIPUnitNotifier != NULL)
-	{
 		fIPUnitNotifier->remove();
-		fIPUnitNotifier = NULL;
-	}
+	
+	fIPUnitNotifier = NULL;
 	
     if(fLcb != NULL)
         IOFree(fLcb, sizeof(LCB));
-
+		
+	fLcb = NULL;
+	
 	if (networkInterface != NULL)
 		networkInterface->release();
+		
+	networkInterface = NULL;
 
 	if(fControl != NULL)
 		fControl->release();
@@ -259,6 +265,11 @@ void IOFireWireIP::stop(IOService *provider)
 
 void IOFireWireIP::free(void)
 {
+    if (ipLock != NULL) 
+        IORecursiveLockFree(ipLock);
+		
+	ipLock = NULL;
+
 	return super::free();
 }
 
@@ -272,15 +283,15 @@ IOReturn IOFireWireIP::message(UInt32 type, IOService *provider, void *argument)
         case kIOMessageServiceIsSuspended:
         case kIOMessageServiceIsResumed:
         case kIOMessageServiceIsRequestingClose:
+			if(busifEnabled)
+				messageClients(type, this);
             res = kIOReturnSuccess;
             break;
-            
+
         default:
             break;
     }
 	
-	messageClients(type, this, 0);
-	
     return res;
 }
 
@@ -304,14 +315,7 @@ IOReturn IOFireWireIP::getMaxPacketSize(UInt32 * maxSize) const
 
 IOReturn IOFireWireIP::getHardwareAddress(IOFWAddress *ea)
 {
-	ea->bytes[0] = macAddr[0];
-	ea->bytes[1] = macAddr[1];
-	ea->bytes[2] = macAddr[2];
-	ea->bytes[3] = macAddr[3];
-	ea->bytes[4] = macAddr[4];
-	ea->bytes[5] = macAddr[5];
-	ea->bytes[6] = macAddr[6];
-	ea->bytes[7] = macAddr[7];
+	memcpy(ea->bytes, macAddr, FIREWIRE_ADDR_LEN);
 
     return kIOReturnSuccess;
 } // end getHardwareAddress
@@ -478,8 +482,6 @@ IOReturn IOFireWireIP::disable(IONetworkInterface * /*netif*/)
 {
     netifEnabled = false;
 
-	// IOLog("IOFireWireIP: disable \n");
-
 	/*
      * Disable our IOOutputQueue object. This will prevent the
      * outputPacket() method from being called.
@@ -652,11 +654,37 @@ void IOFireWireIP::makeEthernetAddress(CSRNodeUniqueID	*fwuid, u_char *bufAddr,
 	getBytesFromGUID(fwuid, bufAddr, GUID_TYPE);
 }
 
-void IOFireWireIP::updateMTU(bool onLynx)
+void IOFireWireIP::updateMTU(UInt32 mtu)
 {
-	// If on Lynx, we assume to do only 512 as the MTU
-	(onLynx) ? networkInterface->setIfnetMTU( 1 << 9 ) :
-			   networkInterface->setIfnetMTU( 1 << fDevice->maxPackLog(true));
+	networkInterface->setIfnetMTU( mtu );
+}
+
+UInt32	IOFireWireIP::getMaxARDMAPacketSize()
+{
+	UInt32	maxARDMASize = 0;
+	
+	OSObject *regProperty = fControl->getProperty("FWARDMAMax", gIOServicePlane);
+	
+	if(regProperty != NULL)
+	{
+		maxARDMASize = ((OSNumber*)regProperty)->unsigned32BitValue();
+		maxARDMASize -= sizeof(IP1394_UNFRAG_HDR);
+	}
+		
+	return maxARDMASize;
+}
+
+UInt8	IOFireWireIP::getMaxARDMARec(UInt32 size)
+{
+    UInt8 maxRecLog	 = 1;
+	
+    while( (size >= 8) && (maxRecLog < 15) )
+    {
+        size >>= 1;
+        maxRecLog++;
+    }
+	
+	return maxRecLog;
 }
 
 void IOFireWireIP::registerFWIPPrivateHandlers(IOFireWireIPPrivateHandlers *privateSelf)
@@ -766,127 +794,25 @@ IOReturn IOFireWireIP::createIPConfigRomEntry()
 }
 
 #ifdef DEBUG
-// Display the reassembly control block
-void IOFireWireIP::showRcb(RCB *rcb) {
-	if (rcb != NULL) {
-      IOLog("RCB %p\n\r", rcb);
-      IOLog(" sourceID %04X dgl %u etherType %04X mBlk %p\n\r", rcb->sourceID, rcb->dgl, rcb->etherType, rcb->mBuf);
-      IOLog(" datagramSize %u residual %u\n\r", rcb->datagramSize, rcb->residual);
-	}
-}
 
-void IOFireWireIP::showArb(ARB *arb) {
-
-   u_char ipAddress[4];
-
-   IOLog("ARB %p\n\r", arb);
-   memcpy(ipAddress, &arb->ipAddress, sizeof(ipAddress));
-   IOLog(" IP address %u.%u.%u.%u EUI-64 %08lX %08lX\n\r", ipAddress[0],
-          ipAddress[1], ipAddress[2], ipAddress[3], arb->eui64.hi,
-          arb->eui64.lo);
-   IOLog(" fwAddr  %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n\r", arb->fwaddr[0],
-          arb->fwaddr[1], arb->fwaddr[2], arb->fwaddr[3], arb->fwaddr[4],
-          arb->fwaddr[5], arb->fwaddr[6], arb->fwaddr[7]);
-   IOLog(" Handle: %08lX %02X %02X %04X%08lX\n\r", arb->handle.unicast.deviceID,
-          arb->handle.unicast.maxRec, arb->handle.unicast.spd,
-          arb->handle.unicast.unicastFifoHi, arb->handle.unicast.unicastFifoLo);
-   IOLog(" Timer %d \n\r", arb->timer);
-
-}
-
-void IOFireWireIP::showHandle(TNF_HANDLE *handle) {
-
-   if (handle->unicast.deviceID != kInvalidIPDeviceRefID)
-      IOLog("   Unicast handle: %08lX %02X %02X %04X%08lX\n\r",
-             handle->unicast.deviceID, handle->unicast.maxRec,
-             handle->unicast.spd, handle->unicast.unicastFifoHi,
-             handle->unicast.unicastFifoLo);
-   else
-      IOLog("   Multicast handle: 00000000 %02X %02X %02X %08lX\n\r",
-             handle->multicast.maxRec, handle->multicast.spd,
-             handle->multicast.channel, htonl(handle->multicast.groupAddress));
-
-}
-
-void IOFireWireIP::showDrb(DRB *drb) 
+void IOFireWireIP::showIPStatus()
 {
-   if (drb != NULL) {
-      IOLog("DRB 0x%p (associated with LCB 0x%p)\n\r", drb, drb->lcb);
-      IOLog(" Device ID %08lX EUI-64 %08lX %08lX\n\r", drb->deviceID, drb->eui64.hi, drb->eui64.lo);
-      IOLog(" timer %08lX maxPayload %d maxSpeed %d\n\r", drb->timer, drb->maxPayload, drb->maxSpeed);
-   }
+	IOLog("	txB %d, rxB %d, txU %d, rxUni %d, rxF %d txF %d qSt %d, qStallCt %d, qRetryCt %d, qGetSize %d, \
+				acCmds %d, noCmds %d, noBCmds %d, inacCmds %d, dc %d, subErr %d, callErr %d, \
+				stall %d, noRes %d, activeMbufs %d inActiveMbufs %d \
+				\n",
+		fTxBcast,
+		fRxBcast,fTxUni,fRxUni, 
+		fRxFragmentPkts,fTxFragmentPkts, transmitQueue->getState(),
+		transmitQueue->getStallCount(), transmitQueue->getRetryCount(), 
+		transmitQueue->getSize(), 
+		fActiveCmds, fNoCommands,fNoBCastCommands, 
+		fInActiveCmds, fDoubleCompletes, 
+		fSubmitErrs, fCallErrs,  
+		fNoResources, activeMbufs, inActiveMbufs
+		);
 }
 
-void IOFireWireIP::showLcb() {
-
-   CBLK *cBlk;
-   UNSIGNED cCBlk = 0;
-
-   IOLog(" Node ID %04X maxPayload %u maxSpeed %u busGeneration 0x%08lX\n\r",
-          fLcb->ownNodeID, fLcb->ownMaxPayload,
-          fLcb->ownMaxSpeed, fLcb->busGeneration);
-   IOLog(" Free CBLKs %u (of %u in pool)\n\r", fLcb->cFreeCBlk,
-          fLcb->nCBlk);
-   IOLog(" CBLK Low water mark %u\n\r", fLcb->minFreeCBlk);
-   
-   // Display the arb's
-   if (fLcb->unicastArb == NULL)
-      IOLog(" No unicast ARBs\n\r");
-   else {
-      IOLog(" Unicast ARBs\n\r");
-      cBlk = (CBLK*)&fLcb->unicastArb;
-      while ((cBlk = cBlk->next) != NULL) {
-         cCBlk++;
-         IOLog("  %p\n\r", cBlk);
-		 showArb((ARB*)cBlk);
-      }
-   }
-   
-   // Display the multicast arb's
-   if (fLcb->multicastArb == NULL)
-      IOLog(" No multicast ARBs\n\r");
-   else {
-      IOLog(" Multicast ARBs\n\r");
-      cBlk = (CBLK*)&fLcb->multicastArb;
-      while ((cBlk = cBlk->next) != NULL) {
-         cCBlk++;
-         IOLog("  %p\n\r", cBlk);
-      }
-   }
-   
-   // Display the active DRB
-   if (fLcb->activeDrb == NULL)
-      IOLog(" No active DRBs\n\r");
-   else {
-      IOLog(" Active DRBs\n\r");
-      cBlk = (CBLK*)&fLcb->activeDrb;
-      while ((cBlk = cBlk->next) != NULL) {
-         cCBlk++;
-         IOLog("  %p\n\r", cBlk);
-		 showDrb((DRB*)cBlk);
-      }
-   }
-   
-   // Display the active RCB
-   if (fLcb->activeRcb == NULL)
-      IOLog(" No active RCBs\n\r");
-   else {
-      IOLog(" Active RCBs\n\r");
-      cBlk = (CBLK*)&fLcb->activeRcb;
-      while ((cBlk = cBlk->next) != NULL) {
-         cCBlk++;
-         IOLog("  %p\n\r", cBlk);
-		 showRcb((RCB*)cBlk);
-      }
-   }
-   
-   IOLog(" %u CBLKs in use\n\r", cCBlk);
-   if (cCBlk + fLcb->cFreeCBlk != fLcb->nCBlk)
-      IOLog(" CBLK accounting error!\n\r");
-
-}
-
-//---------------------------------------------------------------------------
 // Used for debugging only. Log the mbuf fields.
 void _logMbuf(struct mbuf * m)
 {
@@ -944,9 +870,6 @@ void _logPkt(void *pkt, UInt16 len)
 {
 	UInt8 	*bytePtr;
 
-	///
-	// start log code
-	///
 	bytePtr = (UInt8*)pkt;
 	
 	IOLog("pkt {\n") ;
@@ -966,8 +889,5 @@ void _logPkt(void *pkt, UInt16 len)
 		IOLog("%02X", (unsigned char)bytePtr[index]) ;
 	}
 	IOLog("}\n\n") ;
-	//////
-	// end log code
-	//////
 }
 #endif 
